from mnspy.ecuaciones_diferenciales_parciales.mef import Nodo, Elemento, Rigidez
from mnspy.utilidades import es_notebook, _generar_matrix
from IPython.display import display, Math
from tabulate import tabulate
import numpy as np

class TriangularCST(Elemento):
    """Representa un elemento Triangular de Deformación Constante (CST) para análisis 2D.

    Este elemento se utiliza para modelar problemas de **tensión plana**, donde se asume
    que los esfuerzos en la dirección del espesor (z) son cero (σ_z = τ_xz = τ_yz = 0).
    Es la base para el análisis de placas delgadas sometidas a cargas en su plano.

    El nombre "Deformación Constante" se debe a que las deformaciones (y por lo tanto
    los esfuerzos) son uniformes en todo el elemento.

    Cada nodo del elemento tiene dos grados de libertad:
    1. Desplazamiento en la dirección 'x' global.
    2. Desplazamiento en la dirección 'y' global.

    Attributes
    ----------
    _E : float
        Módulo de Young del material.
    _t : float
        Espesor del elemento.
    _coef_poisson : float
        Coeficiente de Poisson del material.
    _A : float
        Área del elemento en el plano xy.
    _B : np.ndarray
        Matriz de deformación-desplazamiento [B], que relaciona las deformaciones
        en el elemento con los desplazamientos nodales: `{ε} = [B]{d}`.
    _D : np.ndarray
        Matriz constitutiva (esfuerzo-deformación) [D] para tensión plana, que
        relaciona los esfuerzos con las deformaciones: `{σ} = [D]{ε}`.
    _k : Rigidez
        Objeto `Rigidez` que contiene la matriz de rigidez del elemento en
        coordenadas globales.
    """

    def __init__(self, nombre: str, nodo_i: Nodo, nodo_j: Nodo, nodo_m: Nodo, E: float, espesor: float,
                 coef_poisson: float = 0.3):
        """Constructor para el elemento Triangular de Deformación Constante.

        Parameters
        ----------
        nombre : str
            Nombre o identificador del elemento.
        nodo_i : Nodo
            Primer nodo del elemento (orden antihorario).
        nodo_j : Nodo
            Segundo nodo del elemento (orden antihorario).
        nodo_m : Nodo
            Tercer nodo del elemento (orden antihorario).
        E : float
            Módulo de Young del material.
        espesor : float
            Espesor del elemento (t).
        coef_poisson : float, optional
            Coeficiente de Poisson (ν). Por defecto es 0.3.
        """
        super().__init__(nombre, nodo_i, nodo_j, nodo_m)
        self._E = E
        self._t = espesor
        self._coef_poisson = coef_poisson
        x_i, y_i, z = nodo_i.punto
        x_j, y_j, z = nodo_j.punto
        x_m, y_m, z = nodo_m.punto

        # --- Cálculo de coeficientes geométricos para las funciones de forma ---
        # alfa_i = x_j * y_m - y_j * x_m
        # alfa_j = y_i * x_m - x_i * y_m
        # alfa_m = x_i * y_j - y_i * x_j
        beta_i = y_j - y_m
        beta_j = y_m - y_i
        beta_m = y_i - y_j
        gamma_i = x_m - x_j
        gamma_j = x_i - x_m
        gamma_m = x_j - x_i
        self._A = (x_i * (y_j - y_m) + x_j * (y_m - y_i) + x_m * (y_i - y_j)) / 2

        # --- Matriz de deformación-desplazamiento [B] ---
        self._B = (1 / (2 * self._A)) * np.array([[beta_i, 0, beta_j, 0, beta_m, 0],
                            [0, gamma_i, 0, gamma_j, 0, gamma_m],
                            [gamma_i, beta_i, gamma_j, beta_j, gamma_m, beta_m]],
                           dtype=np.double)
        # --- Matriz constitutiva [D] para Tensión Plana ---
        self._D = (E / (1 - coef_poisson ** 2)) * np.array([[1, coef_poisson, 0],
                            [coef_poisson, 1, 0],
                            [0, 0, (1 - coef_poisson) / 2]],
                           dtype=np.double)

        # --- Matriz de rigidez [k] = t * A * [B]^T * [D] * [B] ---
        k_matrix = self._t * abs(self._A) * self._B.T @ self._D @ self._B

        self._k = Rigidez(k_matrix,
                          [self._nodo_i, self._nodo_j, self._nodo_m],
                          ['x', 'y'])
        self._fuerzas_i = np.zeros((len(self._k.grados), 1))
        self._fuerzas_j = np.zeros((len(self._k.grados), 1))
        self._fuerzas_m = np.zeros((len(self._k.grados), 1))
        self._nodo_i.grados_libertad['x'].label_reaccion = 'F'
        self._nodo_i.grados_libertad['x'].label_fuerza = 'f'
        self._nodo_i.grados_libertad['y'].label_reaccion = 'F'
        self._nodo_i.grados_libertad['y'].label_fuerza = 'f'
        self._nodo_j.grados_libertad['x'].label_reaccion = 'F'
        self._nodo_j.grados_libertad['x'].label_fuerza = 'f'
        self._nodo_j.grados_libertad['y'].label_reaccion = 'F'
        self._nodo_j.grados_libertad['y'].label_fuerza = 'f'
        self._nodo_m.grados_libertad['x'].label_reaccion = 'F'
        self._nodo_m.grados_libertad['x'].label_fuerza = 'f'
        self._nodo_m.grados_libertad['y'].label_reaccion = 'F'
        self._nodo_m.grados_libertad['y'].label_fuerza = 'f'

    def _repr_latex_(self):
        """Representación en LaTeX del sistema local del elemento para notebooks."""
        vec_d = np.array(self._k.obtener_etiquetas_desplazamientos(), dtype=object).reshape(-1, 1)
        vec_f = np.array(self._obtener_etiquetas_fuerzas(), dtype=object).reshape(-1,1)
        texto_latex = r'\left\{\begin{array}{c}'
        texto_latex += _generar_matrix(vec_f,
                                       '{:}') + r'\end{array}\right\}_{\{f\}}=\left[\begin{array}{' + 'c' * \
                       self._k.obtener_matriz().shape[1] + '}'
        texto_latex += _generar_matrix(self._k.obtener_matriz(),
                                       '{:G}') + r'\end{array}\right]_{[k]}\cdot\left\{\begin{array}{c}'
        texto_latex += _generar_matrix(vec_d, '{:}') + r'\end{array}\right\}_{\{d\}}'
        return '$' + texto_latex + '$'

    def __repr__(self):
        """Representación del objeto como string."""
        return f'TriangularCST: {self.nombre}'

    def __str__(self):
        """Representación del objeto como string."""
        return 'TriangularCST: ' + self.nombre

    def mostrar_sistema(self, reducida: bool = False):
        """Muestra el sistema de ecuaciones del elemento en formato matricial."""
        vec_d = np.array(self._k.obtener_etiquetas_desplazamientos(reducida), dtype=object).reshape(-1, 1)
        vec_f = np.array(self._obtener_etiquetas_fuerzas(reducida), dtype=object).reshape(-1,1)
        if es_notebook():
            texto_latex = r'\left\{\begin{array}{c}'
            texto_latex += _generar_matrix(vec_f,
                                           '{:}') + r'\end{array}\right\}_{\{f\}}=\left[\begin{array}{' + 'c' * \
                           self._k.obtener_matriz(reducida).shape[1] + '}'
            texto_latex += _generar_matrix(self._k.obtener_matriz(reducida),
                                           '{:G}') + r'\end{array}\right]_{[k]}\cdot\left\{\begin{array}{c}'
            texto_latex += _generar_matrix(vec_d, '{:}') + r'\end{array}\right\}_{\{d\}}'
            display(Math(texto_latex))
        else:
            return np.array2string(self._k.obtener_matriz(reducida),
                                   formatter={'float_kind': lambda x: '{:}'.format(x)})

    def _obtener_etiquetas_fuerzas(self, reducida: bool = False) -> list[str]:
        """Genera las etiquetas para las fuerzas nodales del elemento."""
        etq = [nodo.grados_libertad[gl].label_fuerza + '^{(' + self.nombre + ')}_{' + nodo.nombre + gl + '}' for nodo
               in [self._nodo_i, self._nodo_j, self._nodo_m] for gl in self._k.grados if
               not reducida or nodo.grados_libertad[gl].valor]
        return etq

    def _calcular_esfuerzos(self) -> np.ndarray:
        """Calcula el vector de esfuerzos {σ} = [D][B]{d} para el elemento.

        Returns
        -------
        np.ndarray
            Vector de esfuerzos [σ_x, σ_y, τ_xy]^T.
        """
        return np.matmul(np.matmul(self._D, self._B), self._obtener_desplazamientos())

    def _calcular_esfuerzos_principales(self) -> list:
        """Calcula los esfuerzos principales y la orientación del plano principal.

        Se basa en las ecuaciones del círculo de Mohr para tensión plana.

        Returns
        -------
        list[float, float, float]
            Una lista con el esfuerzo principal máximo (σ₁), el esfuerzo principal
            mínimo (σ₂) y el ángulo del plano principal (θ_p) en grados.
        """
        esfuerzos = self._calcular_esfuerzos()
        s_x = esfuerzos[0, 0]
        s_y = esfuerzos[1, 0]
        t_xy = esfuerzos[2, 0]

        # Cálculo de los esfuerzos principales
        a = np.sqrt(((s_x - s_y) / 2) ** 2 + t_xy ** 2)
        s_1 = (s_x + s_y) / 2 + a  # Esfuerzo máximo
        s_2 = (s_x + s_y) / 2 - a  # Esfuerzo mínimo

        # Cálculo del ángulo del plano principal
        if s_x == s_y:
            teta = 45
        else:
            teta = np.degrees(0.5 * np.arctan2(2 * t_xy, (s_x - s_y)))
        return [s_1, s_2, teta]

    def esfuerzos(self):
        """Calcula y muestra los componentes de esfuerzo (σ_x, σ_y, τ_xy).

        Returns
        -------
        str or np.ndarray
            Una tabla HTML para notebooks de Jupyter, o imprime y retorna el
            vector de esfuerzos en otros entornos.
        """
        esfuerzos = self._calcular_esfuerzos()
        if es_notebook():
            indice = ['$' + label + '$' for label in [r'\sigma_{x}', r'\sigma_{y}', r'\tau_{xy}']]
            return tabulate({'Esfuerzos': esfuerzos}, headers='keys', showindex=indice, tablefmt='html')
        else:
            print(esfuerzos)
            return esfuerzos

    def esfuerzos_principales(self):
        """Calcula y muestra los esfuerzos principales (σ_max, σ_min) y el ángulo.

        Returns
        -------
        str or list
            Una tabla HTML para notebooks de Jupyter, o imprime y retorna los
            resultados en una lista.
        """
        s_principales = self._calcular_esfuerzos_principales()
        if es_notebook():
            indice = ['$' + label + '$' for label in [r'\sigma_{max}', r'\sigma_{min}', r'\theta_{p}']]
            return tabulate({'Esfuerzos Principales': s_principales}, headers='keys', showindex=indice, tablefmt='html')
        else:
            print(s_principales)
            return s_principales

    def _calcular_esfuerzo_von_mises(self) -> float:
        """Calcula el esfuerzo equivalente de von Mises para tensión plana.

        El esfuerzo de von Mises es un criterio de fluencia para materiales dúctiles.
        Se asume un estado de tensión plana (σ₃ = 0).

        Returns
        -------
        float
            El valor del esfuerzo de von Mises.
        """
        s_principales = self._calcular_esfuerzos_principales()
        s_1, s_2, a = s_principales
        s_3 = 0
        return np.sqrt((s_1 - s_2) ** 2 + (s_2 - s_3) ** 2 + (s_3 - s_1) ** 2) / np.sqrt(2)

    def esfuerzo_von_mises(self):
        """Calcula y muestra el esfuerzo equivalente de von Mises.

        Returns
        -------
        str or float
            Una tabla HTML para notebooks de Jupyter, o imprime y retorna el valor.
        """
        s_vm = self._calcular_esfuerzo_von_mises()
        if es_notebook():
            indice = [r'$\sigma_{vm}$']
            return tabulate({'Esfuerzo de Von Mises': [s_vm]}, headers='keys', showindex=indice, tablefmt='html')
        else:
            print(s_vm)
            return s_vm

    def _obtener_desplazamientos(self) -> np.ndarray:
        """Recupera los desplazamientos de los nodos del elemento."""
        desplazamiento = None
        for item in self._k.lista_nodos:
            d_i = [d.desplazamiento for d in item.grados_libertad.values() if d.gl in self._k.grados]
            if desplazamiento is None:
                desplazamiento = d_i
            else:
                desplazamiento = np.hstack((desplazamiento, d_i))
        return np.array(desplazamiento).reshape(-1,1)



from mnspy.interpolación import Interpolacion
import numpy as np
import matplotlib.pyplot as plt
from IPython.display import display, Math
plt.rcParams.update(plt.rcParamsDefault)

class SplineLineal(Interpolacion):
    """Calcula la interpolación por trazadores (splines) lineales.

    Este método conecta puntos de datos consecutivos con segmentos de línea recta,
    creando una función definida por tramos. Es la forma más simple de
    interpolación por trazadores.

    Attributes
    ----------
    x : np.ndarray
        Array con los datos de la variable independiente.
    y : np.ndarray
        Array con los datos de la variable dependiente.

    Methods
    -------
    evaluar(x: float):
        Evalúa la interpolación lineal en un punto `x`.
    obtener_polinomio():
        Genera la expresión simbólica de la función a tramos.
    graficar():
        Genera una gráfica de los trazadores lineales.

    Examples
    -------
    from mnspy import SplineLineal
    import numpy as np

    T = np.array([-40., 0., 20., 50., 100, 150, 200, 250, 300, 400, 500])
    rho = np.array([1.52, 1.29, 1.2, 1.09, 0.95, 0.84, 0.75, 0.68, 0.62, 0.53, 0.46])
    sc = SplineLineal(T, rho)
    print(sc.evaluar(350))
    sc.graficar(350)

    x = np.array([1., 4., 5, 6.])
    y = np.log(x)
    inter = SplineLineal(x, y)
    print(inter.evaluar(2))
    inter.graficar(2)
    """
    def __init__(self, x: np.ndarray, y: np.ndarray):
        """Constructor de la clase SplineLineal.

        Parameters
        ----------
        x: np.ndarray
            Array con los datos de la variable independiente
        y: np.ndarray
            Array con los datos de la variable dependiente
        """
        super().__init__(x, y)

    def evaluar(self, x: float) -> float | str:
        """Evalúa el spline lineal en un punto `x`.

        Encuentra el intervalo [x_i, x_{i+1}] que contiene a `x` y aplica la
        fórmula de interpolación lineal en ese segmento.

        Parameters
        ----------
        x: float
            Valor de la variable independiente en el que se desea interpolar.

        Returns
        -------
        float or str
            El valor interpolado y(x), o un mensaje de error si `x` está
            fuera del rango de los datos.
        """
        if (x < self._x[0]) or (x > self._x[self._n - 1]):
            return "Valor de entrada fuera del rango de la tabla"
        i_0 = 0
        for i in range(self._n):
            if x == self._x[i]:
                return float(self._y[i])
            elif self._x[i] > x:
                i_0 = i - 1
                break
        # Fórmula de la recta que pasa por dos puntos
        y_int = self._y[i_0] + (x - self._x[i_0]) * (self._y[i_0 + 1] - self._y[i_0]) / (
                    self._x[i_0 + 1] - self._x[i_0])
        return float(y_int)

    def obtener_polinomio(self):
        """Genera una representación en LaTeX de la función a tramos.

        Returns
        -------
        IPython.display.Math
            Un objeto Math que renderiza la función a tramos en formato LaTeX.
        """
        texto_latex = r'\begin{cases}'
        for i in range(self._n - 1):
            m = (self._y[i + 1] - self._y[i]) / (self._x[i + 1] - self._x[i])
            b = self._y[i] - m * self._x[i]
            sb_0 = '+'
            if b < 0:
                sb_0 = ''
            if m != 0:
                texto_latex += '{:.8G}'.format(m) + 'x'
            if b != 0:
                texto_latex += sb_0 + '{:.8G}'.format(b)
            if i == self._n - 2:
                texto_latex += r' & ' + str(self._x[i]) + r' \leq  x \leq ' + str(self._x[i + 1]) + r'\\'
            else:
                texto_latex += r' & ' + str(self._x[i]) + r' \leq  x < ' + str(self._x[i + 1]) + r'\\'
        texto_latex += r'\end{cases}'
        return display(Math(texto_latex))

    def graficar(self, x: float) -> None:
        """Genera una gráfica de los trazadores lineales.

        Parameters
        ----------
        x: float
            Valor de `x` a interpolar y resaltar en la gráfica.

        Returns
        -------
        None
            Muestra una gráfica de Matplotlib.
        """
        y = self.evaluar(x)
        x_min = float(min(self._x))
        x_max = float(max(self._x))
        x_list = np.linspace(x_min, x_max, 100)
        y_list = list(map(self.evaluar, x_list))
        plt.scatter(x, y, c='r', lw=2, label='Interpolación Spline Lineal')
        plt.plot(x_list, y_list, linestyle='dashed', c='k', lw=1, label='Polinomio')
        plt.annotate('$(' + str(x) + r',\,' + str(y) + ')$', (x, y), c='r', alpha=0.9, textcoords="offset points",
                     xytext=(0, 10), ha='center')
        super()._graficar_datos()


def main():
    T = np.array([-40., 0., 20., 50., 100, 150, 200, 250, 300, 400, 500])
    rho = np.array([1.52, 1.29, 1.2, 1.09, 0.95, 0.84, 0.75, 0.68, 0.62, 0.53, 0.46])
    sc = SplineLineal(T, rho)
    print(sc.evaluar(350))
    sc.graficar(350)

    x = np.array([1., 4., 5, 6.])
    y = np.log(x)
    inter = SplineLineal(x, y)
    print(inter.evaluar(2))
    inter.graficar(2)


if __name__ == '__main__':
    main()



from mnspy.ecuaciones_algebraicas_lineales import EcuacionesAlgebraicasLineales
import numpy as np
from tabulate import tabulate
from mnspy.utilidades import es_notebook


class GaussSediel(EcuacionesAlgebraicasLineales):
    """Clase para la implementación de la solución de un sistema de ecuaciones por el método de Gauss_Jordan.

    Attributes
    ----------
    _A: ndarray
        matrix cuadrada de coeficientes del sistema
    _b: ndarray
        matrix columna de términos independientes
    _etiquetas: dict[str:list[str], str: bool]
        diccionario con etiquetas de la solución
        key: 'label' contiene una lista de etiquetas de la solución
        key: 'es_latex' un boleano que define si el string es en formato Latex o no
    _n: int
        número de filas de la matrix _A
    _m: int
        número de columnas de la matrix _A
    x: ndarray
        matrix columna de la solución del sistema de acuaciones lineales

    Methods
    -------
    _calcular():
        Soluciona el sistema de cuaciones lineales por el método de GaussSediel y la solución queda guardada en el
        atributo x

    solucion():
        Presenta los resultados de la solución del sistema de ecuaciones

    Examples
    -------
    from mnspy import GaussSediel
    import numpy as np

    A = np.matrix('3 -0.1 -0.2;0.1 7 -0.3; 0.3 -0.2 10')
    print('Matrix A:\n', A)
    b = np.matrix('7.85 ; -19.3; 71.4')
    print('Matrix b:\n', b)
    gj = GaussSediel(A, b)
    print('solución:')
    gj.solucion()
    gj.generar_tabla()
    """

    def __init__(self, A: np.ndarray, b: np.ndarray, val_inicial: float = 0.0, tol_porc: float = 0.1,
                 factor_lambda: float = 1.0, iter_max: int = 20):
        """Constructor de la clase GaussSediel

        Parameters
        ----------
        A: ndarray
            Matrix de los coeficientes del sistema de ecuaciones
        b: ndarray
            Vector columna de los terminos independiantes
        val_inicial: float
            Valor inicial de las variables cuando empieza la iteración, por defecto es 0.0
        tol_porc:: float
            Valor con el máximo porcentaje de error permitido, por defecto es 0.1%
        factor_lambda: float
            Valor con el factor de relajamiento que multiplicará la fórmula de iteración, por defecto es 1.0
        iter_max: int
            Corresponde al número máximo de iteraciones permitidas
        """
        super().__init__(A, b)
        self._aumentada = None
        self._iter_max = iter_max
        self._factor_lambda = factor_lambda
        self._tol_porc = tol_porc
        self._converge = True
        self._iter = 0
        self._v_inicial = val_inicial
        self.x = np.full((self._n, 1), self._v_inicial, dtype=float)
        self._last_x = self.x.copy()
        self._iter_valores = self.x.copy()
        self._EA = []
        self._A_mod = self._A.copy()
        np.fill_diagonal(self._A_mod, 0)
        self._dia = np.diag(self._A).reshape(-1, 1)
        if self._n != self._m:
            print('La matriz A debe ser cuadrada')
            return
        if np.count_nonzero(self._dia) != self._n:
            print('La diagonal de A no puede tener ceros')
            return
        self._calcular()

    def _calcular(self):
        """Soluciona el sistema de ecuaciones lineales por el método de Gauss Sediel

        Returns
        -------
        None
        """
        for k in range(self._iter_max):
            for i in range(self._n):
                self.x[i, 0] = self._factor_lambda * (
                        (self._b[i, 0] - np.sum(np.matmul(self._A_mod[i], self.x))) / self._dia[i, 0]) + (
                                       1 - self._factor_lambda) * self.x[i, 0]
            error = max(abs((self.x - self._last_x) / self.x))[0]
            self._last_x = self.x.copy()
            self._converge = error <= self._tol_porc / 100.0
            self._iter_valores = np.hstack((self._iter_valores, self.x))
            self._EA.append(error * 100)
            if self._converge:
                break

    def generar_tabla(self):
        """Genera la tabla de iteraciones

        Returns
        -------
        Tabla de iteraciones usando el pquete tabulate
        """
        valores = np.hstack((self._iter_valores[:, 1:].transpose(), np.array(self._EA).reshape(-1,1)))
        if es_notebook():
            if self._etiquetas is None:
                indice = ['Iteración'] + ['$x_{' + str(i) + '}$' for i in range(self.x.shape[0])] + [
                    '$\\varepsilon_{a}[\\%]$']
            else:
                if self._etiquetas['es_latex']:
                    indice = ['Iteración'] + ['$' + label + '$' for label in self._etiquetas['label']] + [
                        '$\\varepsilon_{a}$']
                else:
                    indice = ['Iteración'] + self._etiquetas['label'] + ['EA[%]']
            return tabulate(np.array(valores), indice, showindex=list(range(1, valores.shape[0] + 1)),
                            tablefmt='html', colalign=("center",))
        else:
            if self._etiquetas is None:
                indice = ['Iteración'] + ['x_' + str(i) for i in range(self.x.shape[0])] + ['EA']
            else:
                indice = ['Iteración'] + self._etiquetas['label'] + ['EA']
            print(tabulate(np.array(valores), indice, showindex=list(range(1, valores.shape[0] + 1)), tablefmt='simple',
                           colalign=("center",)))
            return None


def main():
    A = np.array([[3, -0.1, -0.2], [0.1, 7, -0.3], [0.3, -0.2, 10]])
    print('Matrix A:\n', A)
    b = np.array([[7.85], [-19.3], [71.4]])
    print('Matrix b:\n', b)
    gj = GaussSediel(A, b)
    print('solución:')
    gj.solucion()
    gj.generar_tabla()


if __name__ == '__main__':
    main()



from mnspy.raíces import Raices
import numpy as np
import matplotlib.pyplot as plt
import sys
plt.rcParams.update(plt.rcParamsDefault)

class FalsaPosicion(Raices):
    """Clase para la implementación del cálculo de raíces por el método cerrado de la Falsa Posición.

    Attributes
    ----------
    f: callable
        Función a la que se le hallará la raíz.
    x_min: float
        Límite inferior del intervalo de búsqueda.
    x_max: float
        Límite superior del intervalo de búsqueda.
    tol: float | int
        Máxima tolerancia del error.
    max_iter: int
        Número máximo de iteraciones permitido.
    tipo_error: str
        Tipo de error a utilizar para la convergencia:
        - ``'%'``: Error relativo porcentual.
        - ``'/'``: Error relativo.
        - ``'n'``: Número de cifras significativas.

    Methods
    -------
    _calcular():
        Realiza los cálculos iterativos del método de la Falsa Posición.
    graficar(mostrar_sol: bool, mostrar_iter: bool, mostrar_lin_iter: bool, n_puntos: int):
        Genera una gráfica del proceso de búsqueda de la raíz.

    Examples
    -------
    from mnspy import FalsaPosicion

    def f(x):
        return x ** 3 - 10 * x ** 2 + 5

    fp = FalsaPosicion(f, 0, 1, tol=4, tipo_error='n')
    fp.generar_tabla()
    fp.graficar()

    fp = FalsaPosicion(f, 0, 1, tol=5, tipo_error='%')
    fp.generar_tabla()
    fp.graficar()
    """
    def __init__(self, f: callable, x_min: float = 0, x_max: float = 0, tol: float | int = 1e-3, max_iter: int = 20,
                 tipo_error='%'):
        """
        Constructor de la clase FalsaPosicion.

        Parameters
        ----------
        f: callable
            Función a la que se le hallará la raíz.
        x_min: float
            Límite inferior del intervalo de búsqueda.
        x_max: float
            Límite superior del intervalo de búsqueda.
        tol: float | int
            Máxima tolerancia del error.
        max_iter: int
            Número máximo de iteraciones permitido.
        tipo_error: str
            Tipo de error a utilizar: ``'%'``, ``'/'`` o ``'n'``.
        """
        super().__init__(f, x_min, x_max, tol, max_iter, tipo_error)
        self._calcular()

    def _calcular(self):
        """Calcula la raíz de la función de forma iterativa usando el método de Falsa Posición.

        Este método, también conocido como Regula Falsi, encuentra la raíz
        trazando una línea recta entre los puntos (x_min, f(x_min)) y
        (x_max, f(x_max)) y encontrando la intersección de esta línea con el eje x.

        Returns
        -------
        None
        """
        # Verifica que la raíz se encuentre dentro del intervalo inicial.
        if np.sign(self._f(self._x_min)) == np.sign(self._f(self._x_max)):
            print("La raíz no está dentro de este rango, pruebe con otro rango de datos")
            sys.exit()

        # Bucle de iteración para evitar recursión profunda
        while True:
            self.x = self._x_max - (self._f(self._x_max) * (self._x_min - self._x_max)) / (
                    self._f(self._x_min) - self._f(self._x_max))

            if self._fin_iteracion():
                break  # Convergencia o máximo de iteraciones alcanzado

            if np.sign(self._f(self._x_min)) == np.sign(self._f(self.x)):
                self._x_min = self.x
            else:
                self._x_max = self.x

    def graficar(self, mostrar_sol: bool = True, mostrar_iter: bool = True, mostrar_lin_iter: bool = True,
                 n_puntos: int = 100):
        """Genera una gráfica del proceso de búsqueda de la raíz por Falsa Posición.

        Parameters
        ----------
        mostrar_sol: bool
            Si es ``True``, muestra el punto de la solución final.
        mostrar_iter: bool
            Si es ``True``, muestra los puntos de cada iteración.
        mostrar_lin_iter: bool, optional
            Si es ``True``, muestra las líneas secantes utilizadas en cada
            iteración. Por defecto es ``True``.
        n_puntos: int, optional
            Número de puntos para dibujar la curva de la función. Por defecto es 100.

        Returns
        -------
        None
            Muestra una gráfica de Matplotlib.
        """
        if mostrar_lin_iter and len(self._tabla['x']) > 0:
            for i, x in enumerate(self._tabla['x']):
                y = self._f(x)
                a = self._tabla['x_min'][i]
                b = self._tabla['x_max'][i]
                plt.plot([x, x], [0, y], linestyle='dashed', c='purple', lw=1)
                plt.plot([a, b], [self._f(a), self._f(b)], linestyle='dashed', c='purple', lw=1)
        plt.title(f'x = {self.x}')
        if self._error_porcentual:
            plt.suptitle('Método de la Falsa Posición (tol= ' + str(self._tol) + '%)')
        else:
            plt.suptitle('Método de la Falsa Posición (tol= ' + str(self._tol) + ')')
        super().graficar(mostrar_sol, mostrar_iter, mostrar_lin_iter, n_puntos)


def main():
    """Función principal para demostración."""
    def f(x):
        return x ** 3 - 10 * x ** 2 + 5

    fp = FalsaPosicion(f, 0, 1, tol=4, tipo_error='n')
    fp.generar_tabla()
    fp.graficar()

    fp = FalsaPosicion(f, 0, 1, tol=5, tipo_error='%')
    fp.generar_tabla()
    fp.graficar()


if __name__ == '__main__':
    main()
